@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -25,6 +25,7 @@
 
 #include <errno.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <glib.h>
 #include <gatchat.h>
@@ -44,7 +45,6 @@
 #include <ofono/phonebook.h>
 #include <ofono/sim.h>
 #include <ofono/sms.h>
-#include <ofono/ssn.h>
 #include <ofono/ussd.h>
 #include <ofono/voicecall.h>
 
@@ -78,12 +78,12 @@ static int wavecom_enable(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	device = ofono_modem_get_string(modem, "Device");
-	if (!device)
+	if (device == NULL)
 		return -EINVAL;
 
 	options = g_hash_table_new(g_str_hash, g_str_equal);
 
-	if (!options)
+	if (options == NULL)
 		return -ENOMEM;
 
 	g_hash_table_insert(options, "Baud", "115200");
@@ -95,7 +95,7 @@ static int wavecom_enable(struct ofono_modem *modem)
 
 	g_hash_table_destroy(options);
 
-	if (!channel)
+	if (channel == NULL)
 		return -EIO;
 
 	/*
@@ -108,7 +108,7 @@ static int wavecom_enable(struct ofono_modem *modem)
 	g_at_syntax_unref(syntax);
 	g_io_channel_unref(channel);
 
-	if (!chat)
+	if (chat == NULL)
 		return -ENOMEM;
 
 	if (getenv("OFONO_AT_DEBUG"))
@@ -135,29 +135,44 @@ static int wavecom_disable(struct ofono_modem *modem)
 static void wavecom_pre_sim(struct ofono_modem *modem)
 {
 	GAtChat *chat = ofono_modem_get_data(modem);
+	const char *model;
+	enum ofono_vendor vendor = 0;
+	struct ofono_sim *sim;
 
 	DBG("%p", modem);
 
+	model = ofono_modem_get_string(modem, "Model");
+	if (model && strcmp(model, "Q2XXX") == 0)
+		vendor = OFONO_VENDOR_WAVECOM_Q2XXX;
+
 	ofono_devinfo_create(modem, 0, "atmodem", chat);
-	ofono_sim_create(modem, OFONO_VENDOR_WAVECOM, "atmodem", chat);
+	sim = ofono_sim_create(modem, vendor, "atmodem", chat);
 	ofono_voicecall_create(modem, 0, "atmodem", chat);
+
+	if (vendor == OFONO_VENDOR_WAVECOM_Q2XXX)
+		ofono_sim_inserted_notify(sim, TRUE);
 }
 
 static void wavecom_post_sim(struct ofono_modem *modem)
 {
 	GAtChat *chat = ofono_modem_get_data(modem);
 	struct ofono_message_waiting *mw;
+	const char *model;
+	enum ofono_vendor vendor = 0;
 
 	DBG("%p", modem);
 
+	model = ofono_modem_get_string(modem, "Model");
+	if (model && strcmp(model, "Q2XXX") == 0)
+		vendor = OFONO_VENDOR_WAVECOM_Q2XXX;
+
 	ofono_ussd_create(modem, 0, "atmodem", chat);
 	ofono_call_forwarding_create(modem, 0, "atmodem", chat);
 	ofono_call_settings_create(modem, 0, "atmodem", chat);
 	ofono_netreg_create(modem, 0, "atmodem", chat);
 	ofono_call_meter_create(modem, 0, "atmodem", chat);
 	ofono_call_barring_create(modem, 0, "atmodem", chat);
-	ofono_ssn_create(modem, 0, "atmodem", chat);
-	ofono_sms_create(modem, 0, "atmodem", chat);
+	ofono_sms_create(modem, vendor, "atmodem", chat);
 	ofono_phonebook_create(modem, 0, "atmodem", chat);
 
 	mw = ofono_message_waiting_create(modem);
