@@ -75,7 +75,6 @@ struct ofono_sms {
 	struct ofono_netreg *netreg;
 	unsigned int netreg_watch;
 	unsigned int status_watch;
-	struct ofono_sim *sim;
 	GKeyFile *settings;
 	char *imsi;
 	int bearer;
@@ -1104,23 +1103,35 @@ int __ofono_sms_txq_cancel(struct ofono_sms *sms, const struct ofono_uuid *uuid)
 	return 0;
 }
 
-static GDBusMethodTable sms_manager_methods[] = {
-	{ "GetProperties",    "",    "a{sv}",        sms_get_properties,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SetProperty",      "sv",  "",             sms_set_property,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SendMessage",      "ss",  "o",             sms_send_message,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "GetMessages",       "",    "a(oa{sv})",    sms_get_messages },
+static const GDBusMethodTable sms_manager_methods[] = {
+	{ GDBUS_ASYNC_METHOD("GetProperties",
+				NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+				sms_get_properties) },
+	{ GDBUS_ASYNC_METHOD("SetProperty",
+			GDBUS_ARGS({ "property", "s" }, { "value", "v" }),
+			NULL, sms_set_property) },
+	{ GDBUS_ASYNC_METHOD("SendMessage",
+			GDBUS_ARGS({ "to", "s" }, { "text", "s" }),
+			GDBUS_ARGS({ "path", "o" }),
+			sms_send_message) },
+	{ GDBUS_METHOD("GetMessages",
+			NULL, GDBUS_ARGS({ "messages", "a(oa{sv})" }),
+			sms_get_messages) },
 	{ }
 };
 
-static GDBusSignalTable sms_manager_signals[] = {
-	{ "PropertyChanged",	"sv"		},
-	{ "IncomingMessage",	"sa{sv}"	},
-	{ "ImmediateMessage",	"sa{sv}"	},
-	{ "MessageAdded",	"oa{sv}"	},
-	{ "MessageRemoved",	"o"		},
+static const GDBusSignalTable sms_manager_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
+	{ GDBUS_SIGNAL("IncomingMessage",
+			GDBUS_ARGS({ "message", "s" }, { "info", "a{sv}" })) },
+	{ GDBUS_SIGNAL("ImmediateMessage",
+			GDBUS_ARGS({ "message", "s" }, { "info", "a{sv}" })) },
+	{ GDBUS_SIGNAL("MessageAdded",
+			GDBUS_ARGS({ "path", "o" },
+						{ "properties", "a{sv}" })) },
+	{ GDBUS_SIGNAL("MessageRemoved",
+			GDBUS_ARGS({ "path", "o" })) },
 	{ }
 };
 
@@ -1448,12 +1459,12 @@ static inline gboolean handle_mwi(struct ofono_sms *sms, struct sms *s)
 	return discard;
 }
 
-void ofono_sms_deliver_notify(struct ofono_sms *sms, unsigned char *pdu,
+void ofono_sms_deliver_notify(struct ofono_sms *sms, const unsigned char *pdu,
 				int len, int tpdu_len)
 {
 	struct ofono_modem *modem = __ofono_atom_get_modem(sms->atom);
-	struct ofono_atom *stk_atom;
-	struct ofono_atom *sim_atom;
+	struct ofono_sim *sim;
+	struct ofono_stk *stk;
 	struct sms s;
 	enum sms_class cls;
 
@@ -1522,24 +1533,20 @@ void ofono_sms_deliver_notify(struct ofono_sms *sms, unsigned char *pdu,
 		if (cls != SMS_CLASS_2)
 			break;
 
-		sim_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SIM);
-
-		if (sim_atom == NULL)
+		sim = __ofono_atom_find(OFONO_ATOM_TYPE_SIM, modem);
+		if (sim == NULL)
 			return;
 
-		if (!__ofono_sim_service_available(
-					__ofono_atom_get_data(sim_atom),
+		if (!__ofono_sim_service_available(sim,
 					SIM_UST_SERVICE_DATA_DOWNLOAD_SMS_PP,
 					SIM_SST_SERVICE_DATA_DOWNLOAD_SMS_PP))
 			return;
 
-		stk_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_STK);
-
-		if (stk_atom == NULL)
+		stk = __ofono_atom_find(OFONO_ATOM_TYPE_STK, modem);
+		if (stk == NULL)
 			return;
 
-		__ofono_sms_sim_download(__ofono_atom_get_data(stk_atom),
-						&s, NULL, sms);
+		__ofono_sms_sim_download(stk, &s, NULL, sms);
 
 		/*
 		 * Passing the USIM response back to network is not
@@ -1599,7 +1606,7 @@ out:
 	handle_deliver(sms, &s);
 }
 
-void ofono_sms_status_notify(struct ofono_sms *sms, unsigned char *pdu,
+void ofono_sms_status_notify(struct ofono_sms *sms, const unsigned char *pdu,
 				int len, int tpdu_len)
 {
 	struct sms s;
@@ -1952,7 +1959,7 @@ void ofono_sms_register(struct ofono_sms *sms)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	struct ofono_modem *modem = __ofono_atom_get_modem(sms->atom);
 	const char *path = __ofono_atom_get_path(sms->atom);
-	struct ofono_atom *sim_atom;
+	struct ofono_sim *sim;
 
 	if (!g_dbus_register_interface(conn, path,
 					OFONO_MESSAGE_MANAGER_INTERFACE,
@@ -1974,18 +1981,17 @@ void ofono_sms_register(struct ofono_sms *sms)
 					OFONO_ATOM_TYPE_NETREG,
 					netreg_watch, sms, NULL);
 
-	sim_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SIM);
+	sim = __ofono_atom_find(OFONO_ATOM_TYPE_SIM, modem);
 
 	/*
 	 * If we have a sim atom, we can uniquely identify the SIM,
 	 * otherwise create an sms assembly which doesn't backup the fragment
 	 * store.
 	 */
-	if (sim_atom) {
+	if (sim) {
 		const char *imsi;
 
-		sms->sim = __ofono_atom_get_data(sim_atom);
-		imsi = ofono_sim_get_imsi(sms->sim);
+		imsi = ofono_sim_get_imsi(sim);
 		sms->assembly = sms_assembly_new(imsi);
 
 		sms->sr_assembly = status_report_assembly_new(imsi);
